package org.expeditee.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.expeditee.io.Conversion;
import org.expeditee.items.Item;
import org.expeditee.items.Justification;
import org.expeditee.items.Permission;
import org.expeditee.items.Text;

/**
 * This class provides the methods to extract and set attributes of Items and
 * Frames. These methods are called when a user merges a text item with
 * <code>Attribute: Value</code> pairs.
 * 
 * @author jdm18
 * 
 */
public class AttributeUtils {

	private static final int GET_LENGTH = "get".length();

	private static int SET_LENGTH = "set".length();

	// List of method names to ignore when searching for a match
	private static List<Method> _GetMethods = null;

	private static HashMap<String, Method> _SetMethods = null;

	// List of attributes which are ignored when copying.
	private static List<String> _Ignore = null;

	/***************************************************************************
	 * List of method names to show in extraced lists even when they return null
	 * (Null is often used to indicate the default value is used)
	 **************************************************************************/
	private static List<Method> _AllowNull = null;

	private static List<String> _ExtractIgnore = null;

	// private static HashMap<String, String> _Abbreviations = null;

	/**
	 * Initialises the _Ignore and _AllowNull lists.
	 */
	private static void initLists() {

		Class[] param = {};

		try {
			// TODO load these in with reflection...
			// Set the shortcuts with annotation tags on the methods
			_Ignore = new LinkedList<String>();
			_Ignore.add("date");
			_Ignore.add("datecreated");
			_Ignore.add("d");
			_Ignore.add("link");
			_Ignore.add("l");
			_Ignore.add("action");
			_Ignore.add("a");
			_Ignore.add("position");
			_Ignore.add("pos");
			_Ignore.add("p");
			_Ignore.add("x");
			_Ignore.add("y");			

			_ExtractIgnore = new LinkedList<String>();
			_ExtractIgnore.add("x");
			_ExtractIgnore.add("y");

			_AllowNull = new LinkedList<Method>();
			_AllowNull.add(Item.class.getMethod("getColor", param));
			_AllowNull.add(Item.class.getMethod("getBackgroundColor", param));

			_AllowNull.add(Frame.class.getMethod("getBackgroundColor", param));
			_AllowNull.add(Frame.class.getMethod("getForegroundColor", param));

			_GetMethods = new LinkedList<Method>();
			_GetMethods.add(Item.class.getMethod("getDateCreated", param));

			_GetMethods.add(Item.class.getMethod("getColor", param));
			_GetMethods.add(Item.class.getMethod("getBorderColor", param));
			_GetMethods.add(Item.class.getMethod("getBackgroundColor", param));
			_GetMethods.add(Item.class.getMethod("getAction", param));
			_GetMethods.add(Item.class.getMethod("getData", param));
			_GetMethods.add(Item.class.getMethod("getLink", param));
			_GetMethods.add(Item.class.getMethod("getFormula", param));
			_GetMethods.add(Item.class.getMethod("getFillColor", param));
			_GetMethods.add(Item.class.getMethod("getGradientColor", param));
			_GetMethods.add(Item.class.getMethod("getFillPattern", param));
			_GetMethods.add(Item.class.getMethod("getThickness", param));

			_GetMethods.add(Item.class.getMethod("getOwner", param));
			_GetMethods.add(Item.class.getMethod("getLinkMark", param));
			_GetMethods.add(Item.class.getMethod("getActionMark", param));

			_GetMethods
					.add(Item.class.getMethod("getActionCursorEnter", param));
			_GetMethods
					.add(Item.class.getMethod("getActionCursorLeave", param));
			_GetMethods.add(Item.class.getMethod("getActionEnterFrame", param));
			_GetMethods.add(Item.class.getMethod("getActionLeaveFrame", param));

			_GetMethods.add(Item.class.getMethod("getTopShadowColor", param));
			_GetMethods
					.add(Item.class.getMethod("getBottomShadowColor", param));
			_GetMethods.add(Item.class.getMethod("getArrow", param));

			_GetMethods.add(Item.class.getMethod("getLinePattern", param));
			_GetMethods.add(Item.class.getMethod("getLinkFrameset", param));
			_GetMethods.add(Item.class.getMethod("getLinkTemplate", param));
			_GetMethods.add(Item.class.getMethod("getPosition", param));
			_GetMethods.add(Item.class.getMethod("getX", param));
			_GetMethods.add(Item.class.getMethod("getY", param));

			_GetMethods.add(Item.class.getMethod("getAnchorRight", param));
			_GetMethods.add(Item.class.getMethod("getAnchorBottom", param));
			_GetMethods.add(Item.class.getMethod("getSize", param));

			_GetMethods.add(Text.class.getMethod("getFamily", param));
			_GetMethods.add(Text.class.getMethod("getFontStyle", param));
			_GetMethods.add(Text.class.getMethod("getJustification", param));
			_GetMethods.add(Text.class.getMethod("getWidth", param));

			_GetMethods.add(Frame.class.getMethod("getOwner", param));
			_GetMethods.add(Frame.class.getMethod("getPermission", param));
			_GetMethods.add(Frame.class.getMethod("getDateCreated", param));
			_GetMethods.add(Frame.class.getMethod("getLastModifyUser", param));
			_GetMethods.add(Frame.class.getMethod("getLastModifyDate", param));
			_GetMethods.add(Frame.class.getMethod("getForegroundColor", param));
			_GetMethods.add(Frame.class.getMethod("getBackgroundColor", param));

			Class[] pPoint = { Point.class };
			Class[] pString = { String.class };
			Class[] pInt = { int.class };
			Class[] pFloat = { float.class };
			Class[] pFloatO = { Float.class };
			Class[] pColor = { Color.class };
			Class[] pBool = { boolean.class };
			Class[] pArrow = { float.class, double.class };
			Class[] pList = { List.class };
			Class[] pIntArray = { int[].class };
			Class[] pJustification = { Justification.class };
			Class[] pPermission = { Permission.class };

			// TODO automatically load these by annotating the methods with the
			// shortcut text
			// Indicate to users of any symbols that clash

			_SetMethods = new HashMap<String, Method>();
			_SetMethods.put("x", Item.class.getMethod("setX", pFloat));
			_SetMethods.put("y", Item.class.getMethod("setY", pFloat));
			_SetMethods.put("position", Item.class.getMethod("setPosition",
					pPoint));

			_SetMethods.put("ab", Item.class.getMethod("setAnchorBottom",
					pFloatO));
			_SetMethods.put("ar", Item.class.getMethod("setAnchorRight",
					pFloatO));
			_SetMethods.put("anchorbottom", Item.class.getMethod(
					"setAnchorBottom", pFloatO));
			_SetMethods.put("anchorright", Item.class.getMethod(
					"setAnchorRight", pFloatO));

			_SetMethods.put("p", Item.class.getMethod("setPosition", pPoint));
			_SetMethods.put("pos", Item.class.getMethod("setPosition", pPoint));
			_SetMethods.put("thickness", Item.class.getMethod("setThickness",
					pFloat));
			_SetMethods.put("t", Item.class.getMethod("setThickness", pFloat));

			_SetMethods.put("color", Item.class.getMethod("setColor", pColor));
			_SetMethods.put("c", Item.class.getMethod("setColor", pColor));

			_SetMethods.put("backgroundcolor", Item.class.getMethod(
					"setBackgroundColor", pColor));
			_SetMethods.put("bgc", Item.class.getMethod("setBackgroundColor",
					pColor));
			_SetMethods.put("bordercolor", Item.class.getMethod(
					"setBorderColor", pColor));
			_SetMethods.put("bc", Item.class
					.getMethod("setBorderColor", pColor));

			_SetMethods
					.put("action", Item.class.getMethod("setActions", pList));
			_SetMethods.put("a", Item.class.getMethod("setActions", pList));
			_SetMethods.put("d", Item.class.getMethod("setData", pList));
			_SetMethods.put("data", Item.class.getMethod("setData", pList));

			_SetMethods.put("link", Item.class.getMethod("setLink", pString));
			_SetMethods.put("l", Item.class.getMethod("setLink", pString));

			_SetMethods.put("formula", Item.class.getMethod("calculate",
					pString));
			_SetMethods.put("f", Item.class.getMethod("calculate", pString));

			_SetMethods.put("fillcolor", Item.class.getMethod("setFillColor",
					pColor));
			_SetMethods.put("fc", Item.class.getMethod("setFillColor", pColor));
			_SetMethods.put("gc", Item.class.getMethod("setGradientColor",
					pColor));
			_SetMethods.put("gradientcolor", Item.class.getMethod(
					"setGradientColor", pColor));

			_SetMethods.put("fillpattern", Item.class.getMethod(
					"setFillPattern", pString));
			_SetMethods.put("fp", Item.class.getMethod("setFillPattern",
					pString));

			_SetMethods.put("owner", Item.class.getMethod("setOwner", pString));
			_SetMethods.put("linkmark", Item.class.getMethod("setLinkMark",
					pBool));
			_SetMethods.put("lm", Item.class.getMethod("setLinkMark", pBool));
			_SetMethods.put("actionmark", Item.class.getMethod("setActionMark",
					pBool));
			_SetMethods.put("am", Item.class.getMethod("setActionMark", pBool));

			_SetMethods.put("actioncursorenter", Item.class.getMethod(
					"setActionCursorEnter", pList));
			_SetMethods.put("actioncursorleave", Item.class.getMethod(
					"setActionCursorLeave", pList));
			_SetMethods.put("actionenterframe", Item.class.getMethod(
					"setActionEnterFrame", pList));
			_SetMethods.put("actionleaveframe", Item.class.getMethod(
					"setActionLeaveFrame", pList));

			_SetMethods.put("topshadow", Item.class.getMethod(
					"setTopShadowColor", pColor));
			_SetMethods.put("bottomshadow", Item.class.getMethod(
					"setBottomShadowColor", pColor));
			_SetMethods.put("arrow", Item.class.getMethod("setArrow", pArrow));

			_SetMethods.put("linepattern", Item.class.getMethod(
					"setLinePattern", pIntArray));
			_SetMethods.put("lp", Item.class.getMethod("setLinePattern",
					pIntArray));

			_SetMethods.put("linkframeset", Item.class.getMethod(
					"setLinkFrameset", pString));
			_SetMethods.put("lf", Item.class.getMethod("setLinkFrameset",
					pString));
			_SetMethods.put("linktemplate", Item.class.getMethod(
					"setLinkTemplate", pString));
			_SetMethods.put("lt", Item.class.getMethod("setLinkTemplate",
					pString));

			_SetMethods.put("family", Text.class
					.getMethod("setFamily", pString));
			_SetMethods.put("face", Text.class.getMethod("setFontStyle",
					pString));
			_SetMethods.put("fontstyle", Text.class.getMethod("setFontStyle",
					pString));
			_SetMethods.put("justification", Text.class.getMethod(
					"setJustification", pJustification));
			_SetMethods.put("j", Text.class.getMethod("setJustification",
					pJustification));
			_SetMethods.put("width", Text.class.getMethod("setWidth", pInt));
			_SetMethods.put("size", Item.class.getMethod("setSize", pFloat));
			_SetMethods.put("s", Item.class.getMethod("setSize", pFloat));

			_SetMethods.put("foregroundcolor", Frame.class.getMethod(
					"setForegroundColor", pColor));
			_SetMethods.put("fgc", Frame.class.getMethod("setForegroundColor",
					pColor));
			_SetMethods.put("backgroundcolor0", Frame.class.getMethod(
					"setBackgroundColor", pColor));
			_SetMethods.put("bgc0", Frame.class.getMethod("setBackgroundColor",
					pColor));
			_SetMethods.put("permission", Frame.class.getMethod(
					"setPermission", pPermission));

		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Extracts a list of attributes from the given Object. Any method that
	 * starts with <code>get</code>, takes no arguments and is not found in
	 * the Ignore list will be run, All the attributes are then put into a Text
	 * Item of the form <Name>:<Value> If the value returned by the get method
	 * is null, then the attribute will not be included, unless the name of the
	 * method is found in the AllowNull list.
	 * 
	 * @param toExtract
	 *            The Object from which to extract the attributes
	 * @return A Text Item containing the extracted Attributes.
	 */
	public static Item extractAttributes(Object toExtract) {
		if (toExtract == null)
			return null;

		// ensure the lists are populated
		if (_Ignore == null)
			initLists();

		// StringBuffer to store all the extracted Attribute:Value pairs
		StringBuffer attributes = new StringBuffer();

		// iterate through the list of methods
		for (Method m : _GetMethods) {

			// Make sure the classes of the methods match the item
			if (m.getDeclaringClass().isAssignableFrom(toExtract.getClass())) {
				try {
					Object o = m.invoke(toExtract, (Object[]) null);

					if (o == null) {
						// methods that return null are only included if they
						// are in the AllowNull list
						if (_AllowNull.contains(m)) {
							String name = m.getName().substring(GET_LENGTH)
									.toLowerCase();
							if (name.equals("color"))
								o = "default";
							else if (name.equals("backgroundcolor"))
								o = "transparent";
							else if (name.equals("foregroundcolor"))
								o = "auto";
							else
								o = "";
						} else {
							continue;
						}
					}
					// skip methods that are in the ignore lists
					if (_ExtractIgnore.contains(m.getName().substring(
							GET_LENGTH).toLowerCase())) {
						continue;
					}

					if (o instanceof Integer) {
						Integer i = (Integer) o;
						if (i == Item.DEFAULT_INTEGER)
							continue;
						if (m.getName().endsWith("Justification")
								&& ((Justification) o).toString() != null)
							o = ((Justification) o).toString();
						// -1 indicates default value
						else
							o = i;
					} else if (o instanceof Float) {
						if (((Float) o) < -0.0001)
							continue;
						//Null indicates default
						o = Math.round((Float) o);
					} else if (o instanceof Double) {
						// -1 indicates default value
						if (((Double) o) < 0.0001)
							continue;
					} else if (o instanceof Color) {
						// converts the color to the Expeditee code
						o = Conversion.getExpediteeColorCode((Color) o);
						if (o == null)
							continue;
					} else if (o instanceof Point) {
						Point p = (Point) o;
						o = Math.round(p.getX()) + " " + Math.round(p.getY());
					} else if (o instanceof Font) {
						Font f = (Font) o;

						String s = f.getName() + "-";
						if (f.isPlain())
							s += "Plain";

						if (f.isBold())
							s += "Bold";

						if (f.isItalic())
							s += "Italic";

						s += "-" + f.getSize();
						o = s;
					} else if (o instanceof Text) {
						o = ((Text) o).getFirstLine();
					} else if (o instanceof List) {
						List list = (List) o;
						for (Object ob : list)
							attributes
									.append(m.getName().substring(GET_LENGTH))
									.append(AttributeValuePair.SEPARATOR_STRING)
									.append(ob).append('\n');
						continue;
					} else if (o instanceof int[]) {
						StringBuffer sb = new StringBuffer();
						int[] values = (int[]) o;
						for (int i = 0; i < values.length; i++) {
							sb.append(values[i]).append(' ');
						}
						sb.deleteCharAt(sb.length() - 1);
						o = sb.toString();
					} else if (o instanceof Boolean) {
						// true is the default for boolean values
						if (((Boolean) o).booleanValue())
							continue;
					}
					// Append the attributes
					attributes.append(m.getName().substring(GET_LENGTH))
							.append(AttributeValuePair.SEPARATOR_STRING)
							.append(o).append('\n');
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		// if no attributes were extracted
		if (attributes.length() <= 0)
			return null;

		while (attributes.charAt(attributes.length() - 1) == '\n')
			attributes.delete(attributes.length() - 1, attributes.length());

		// create the text Item
		Frame current = DisplayIO.getCurrentFrame();
		Item attribs = current.getStatsTextItem(attributes.toString());
		return attribs;
	}

	/**
	 * Attempts to set the attribute in the given attribute: value pair. The
	 * value string should be formatted as follows:
	 * <code> Attribute: Value </code> Multiple values can be used if they are
	 * separated by spaces
	 * 
	 * @param toSet
	 *            The Item or Frame to set the attribute of
	 * @param attribs
	 *            The Text item that contains the list of attributes to set
	 * @return True if the attribute(s) were sucessfully set, false otherwise
	 */
	public static boolean setAttribute(Object toSet, Text attribs) {
		return setAttribute(toSet, attribs, 1);
	}
	
	public static boolean setAttribute(Object toSet, Text attribs, int minAttributeLength) {
		// error checking
		if (toSet == null || attribs == null)
			return false;

		if (_Ignore == null)
			initLists();
		
		if(attribs.isAnnotation())
			return false;

		// get the list of attribute: value pairs
		List<String> values = attribs.getTextList();
		// if no pairs exist, we are done
		if (values == null || values.size() == 0) {
			return false;
		}

		// loop through all attribute: value pairs
		for (int i = 0; i < values.size(); i++) {
			AttributeValuePair avp = new AttributeValuePair(values.get(i), false);

			// If the first is not an attribute value pair then dont do
			// attrubute merging
			if (!avp.hasAttribute() || avp.getAttribute().length() < minAttributeLength)
				return false;

			// check if the next string is another attribute to merge or a
			// continuation
			for (; i < values.size() - 1; i++) {
				AttributeValuePair nextAvp = new AttributeValuePair(values
						.get(i + 1), false);

				// if the next String has a colon, then it may be another
				// attribute
				if (nextAvp.hasAttribute()) {
					// if the attribute is the same as v, then it is a
					// continuation
					if (nextAvp.getAttribute().equals(avp.getAttribute())) {
						// strip the attribute from next
						avp.appendValue(nextAvp.getValue());

						// if the attribute is not the same, then it may be a
						// new method
					} else {
						break;
					}
				}

				// v.append("\n").append(next);
			}

			if (!setAttribute(toSet, avp, values.size() > 1)) {

				String stripped = avp.getAttribute();
				if (!avp.hasPair()) {
					// This happens when there is an attribute at the start
					// Then a bunch of plain text
					return false;
				} else if (_Ignore.contains(stripped)) {
					return false;
				} else if (!(_SetMethods.containsKey(stripped))) {
					return false;
				} else {
					String types = "";
					for (Class c : _SetMethods.get(stripped)
							.getParameterTypes())
						types += c.getSimpleName() + " ";
					MessageBay.warningMessage("Wrong arguments for: '"
							+ avp.getAttribute() + "' expecting "
							+ types.trim() + " found '" + avp.getValue() + "'");
				}
			}
		}

		return true;
	}

	/**
	 * Sets a single attrubute of a frame or item.
	 * 
	 * @param toSet
	 * @param avp
	 * @param isAttributeList
	 *            some properties are ignored when attribute list are injected
	 *            into an item. These properties are ignored if this param is
	 *            true
	 * @return
	 */
	private static boolean setAttribute(Object toSet, AttributeValuePair avp,
			boolean isAttributeList) {

		assert (avp.hasAttribute());

		// separate attribute and value from string
		String attribute = avp.getAttribute().toLowerCase();

		String value = avp.getValue();
		assert (value != null);

		// Some properties are ignored when multiple attributes are being set on
		// an item at the same time
		if (isAttributeList && _Ignore.contains(attribute)) {
			// System.out.println("Attribute ignored: " + attribute);
			return true;
		}

		// Separate multiple values if required
		Method toRun = _SetMethods.get(attribute);

		// if this is not the name of a method, it may be the name of an agent
		if (toRun == null) {
			// System.out.println("Attrib not found for: " + attribute);
			return false;
		}

		// if there are duplicate methods with the same name
		List<Method> possibles = new LinkedList<Method>();
		if (toRun.getDeclaringClass().isInstance(toSet))
			possibles.add(toRun);
		int i = 0;
		while (_SetMethods.containsKey(attribute + i)) {
			if (_SetMethods.get(attribute + i).getDeclaringClass()
					.isAssignableFrom(toSet.getClass()))
				possibles.add(_SetMethods.get(attribute + i));
			i++;
		}

		for (Method possible : possibles) {
			Object current = null;
			Object[] param = {};
			// find the corresponding get method for this set method
			// and get the current value of the attribute
			for (Method m : _GetMethods) {
				if (m.getDeclaringClass().isAssignableFrom(toSet.getClass())
						&& m.getName().substring(GET_LENGTH).equals(
								possible.getName().substring(SET_LENGTH))) {
					try {
						current = m.invoke(toSet, param);
					} catch (IllegalArgumentException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (InvocationTargetException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					break;
				}

			}

			try {
				Object[] params = Conversion.Convert(possible, value, current);

				try {
					possible.invoke(toSet, params);
					return true;
				} catch (IllegalArgumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					MessageBay.displayMessage(toSet.getClass().getSimpleName()
							+ " type does not support that attribute.");
					// e.printStackTrace();
				}
			} catch (NumberFormatException e) {

			}
		}

		return false;
	}

	/**
	 * Replaces the current value for the text item with the new value.
	 * 
	 * @param text
	 *            the item whos value is to be changed
	 * @param newValue
	 *            the new value for the item
	 */
	public static void replaceValue(Text text, String newValue) {
		assert (newValue != null);

		AttributeValuePair avp = new AttributeValuePair(text.getText());

		if (avp.getAttribute() == null) {
			avp.setAttribute(avp.getValue());
		}
		avp.setValue(newValue);
		text.setText(avp.toString());
	}
}
