package org.expeditee.gui;

import java.io.File;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;

import org.expeditee.items.Text;

/**
 * Central class to contain all values that can be set by the user on their
 * profile frame. These values should be updated only by
 * FrameUtils.ParseProfile.
 */
public abstract class UserSettings {

	public static int Gravity = 3;

	public static int LineStraightenThreshold = 15;

	public static int NoOpThreshold = 60;
	
	public static int TitlePosition = 150;

	public static int InitialWidth = 1024;
	
	public static int InitialHeight = 768;
	
	public static String Username;

	public static String DefaultFrame = null;

	public static String MenuFrame = null;

	public static String FirstFrame = null;

	public static List<String> FrameDirs = new LinkedList<String>();

	public static List<String> ImageDirs = new LinkedList<String>();

	public static boolean AntiAlias = false;

	public static boolean LineHighlight = false;

	public static boolean Logging = false;

	public static boolean LogStats = true;

	public static boolean Threading = true;

	public static Text ItemTemplate = new Text("@ItemTemplate");
	
	public static Text DotTemplate = new Text("@DotTemplate");

	public static Text AnnotationTemplate = null;

	public static Text CodeCommentTemplate = null;
	
	public static Text TitleTemplate = null;

	public static Text StatTemplate = null;

	// add default values
	public static void Init() {
		String jarPath = getJarLocation();
		if (jarPath != null)
			FrameIO.changeParentFolder(jarPath);
		else
			FrameIO.changeParentFolder("." + File.separator + "expeditee"
					+ File.separator);

		UserSettings.FrameDirs.add(FrameIO.FRAME_PATH);
		UserSettings.FrameDirs.add(FrameIO.PROFILE_PATH);
		UserSettings.FrameDirs.add(FrameIO.HELP_PATH);
		UserSettings.ImageDirs.add(FrameIO.IMAGES_PATH);
	}

	/**
	 * Find where abouts the Jar is located that is running expeditee. The
	 * reason for this was so that the framesets could be stored in the same
	 * location as the jar file for now. Using the current directory fails to
	 * achieve this as the current directory not the same as the directory the
	 * jar is stored in if the user double clicks on the jar file in Linux or
	 * runs the jar from the command line from another directory.
	 * 
	 * @return
	 */
	public static String getJarLocation() {
		// Get a File object for the package
		File directory = null;

		ClassLoader cld = Thread.currentThread().getContextClassLoader();
		URL resource = cld.getResource("org");
		directory = new File(resource.getFile());

		int splitPoint = directory.getPath().indexOf('!');
		if (splitPoint > 0) {
			String jarName = directory.getPath().substring("file:".length(),
					splitPoint);
			// Windows HACK
			if (jarName.indexOf(":") >= 0)
				jarName = jarName.substring(1);

			if (jarName.indexOf("%20") > 0) {
				jarName = jarName.replace("%20", " ");
			}

			int lastFileSeparator = jarName.lastIndexOf(File.separator);
			return jarName.substring(0, lastFileSeparator + 1);
		}
		return null;
	}
}
